﻿using RsaWebapp.Shared.Entities;
using System.Diagnostics;
using System.Security.Cryptography;

namespace RsaWebapp.Shared.Services
{
	public class ImageEncryptorService : IImageEncryptorService
    {
        public const int KEY_LENGTH = 512;
        public const int CHUNK_SIZE = 22;
        public const int BMP_HEADER_SIZE = 54;

        public ImageEncryptorService() { }
        public byte[] DecryptImage(byte[] data, RsaKeys rsaKeys, out long miliseconds)
        {

            byte[] result = new byte[0];
            Stopwatch stopwatch = new Stopwatch();

            using(RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(KEY_LENGTH))
            {
                stopwatch.Start();
				rsa.FromXmlString(rsaKeys.PrivateKeyXml);
				if (data.Length < CHUNK_SIZE)
				{
					result = rsa.Decrypt(data, false);
				}
				else
                {
					byte[] header = new byte[BMP_HEADER_SIZE];
					Array.Copy(data, header, BMP_HEADER_SIZE);
					result = result.Concat(header).ToArray();
					int encryptedChunkSize = KEY_LENGTH / 8;
					for (int i = BMP_HEADER_SIZE; i < data.Length; i += encryptedChunkSize)
					{
						int size = Math.Min(encryptedChunkSize, data.Length - i);
						byte[] chunk = new byte[size];
						Array.Copy(data, i, chunk, 0, size);
						byte[] decryptedChunk = rsa.Decrypt(chunk, true);
						result = result.Concat(decryptedChunk).ToArray();
					}
				}
			    stopwatch.Stop();
            }

            miliseconds = stopwatch.ElapsedMilliseconds;
            return result;

        }

        public byte[] EncryptImage(byte[] data, RsaKeys rsaKeys, out long miliseconds)
        {
			byte[] result = new byte[0];
			Stopwatch stopwatch = new Stopwatch();

			using (RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(KEY_LENGTH))
            {
                stopwatch.Start();
                rsa.FromXmlString(rsaKeys.PublicKeyXml);
                if (data.Length < CHUNK_SIZE)
                {
                    result = rsa.Encrypt(data, false);
                }
				else
				{
					byte[] header = new byte[BMP_HEADER_SIZE];
					Array.Copy(data, header, BMP_HEADER_SIZE);
					result = result.Concat(header).ToArray();
					for (int i = BMP_HEADER_SIZE; i < data.Length; i += CHUNK_SIZE)
					{
						int size = Math.Min(CHUNK_SIZE, data.Length - i);
						byte[] chunk = new byte[size];
						Array.Copy(data, i, chunk, 0, size);
						byte[] encryptedChunk = rsa.Encrypt(chunk, true);
						result = result.Concat(encryptedChunk).ToArray();
					}
				}
				stopwatch.Stop();
                miliseconds = stopwatch.ElapsedMilliseconds;
            }
            return result;
        }
    }
}
