﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using RsaWebapp.Shared.Entities;
using RsaWebapp.Shared.Services;

namespace RsaWebapp.Components.Controllers
{
	[Route("api/[controller]")]
	[ApiController]
	public class UserController : Controller
	{
		private readonly IUserService _userService;

		public UserController(IUserService userService)
		{
			_userService = userService;
		}


        [HttpPost("register")]
		public async Task<IActionResult> Register([FromBody] RegisterResource resource)
		{
			try
			{
				var response = await _userService.Register(resource);
				return Ok(response);
			}
			catch (Exception e)
			{
				return BadRequest(new { ErrorMessage = e.Message });
			}
		}

		[HttpPost("login")]
		public async Task<IActionResult> Login([FromBody] LoginResource resource)
		{
			try
			{
				var response = await _userService.Login(resource);
				return Ok(response);
			}
			catch (Exception e)
			{
				return BadRequest(new { ErrorMessage = e.Message });
			}
		}

		[Authorize("LoggedUserOnly")]
		[Route("/logout")]
		public async Task Logout()
		{
			var prop = new AuthenticationProperties()
			{
				RedirectUri = "/",
			};
			await HttpContext.SignOutAsync(CookieAuthenticationDefaults.AuthenticationScheme, prop);
			
			
		}



		
	}
}
