using Blazored.LocalStorage;
using RsaWebapp.Components;
using RsaWebapp.Shared.Services;
using RsaWebapp.Shared.Data;
using Microsoft.EntityFrameworkCore;
using Microsoft.AspNetCore.Authentication.Cookies;
using System.Security.Claims;
namespace RsaWebapp
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);

            // Add services to the container.
            builder.Services.AddRazorComponents()
                .AddInteractiveServerComponents()
                .AddInteractiveWebAssemblyComponents();
            builder.Services.AddSwaggerGen();
            builder.Services.AddControllers();
            builder.Services.AddBlazoredLocalStorage();
            builder.Services.AddBlazorBootstrap();
            builder.Services.AddScoped(http => new HttpClient
            {
                BaseAddress = new Uri(builder.Configuration.GetSection("BaseApiUri").Value!),
            });
			builder.Services.Configure<FpgaConfigurationService>(
			   builder.Configuration.GetSection("FpgaConfigurationService"));
			builder.Services.AddScoped<IUartService, UartService>();
            builder.Services.AddScoped<IRsaService, RsaService>();
            builder.Services.AddScoped<IUserService, UserService>();
            builder.Services.AddScoped<IImageEncryptorService, ImageEncryptorService>();
            builder.Services.AddScoped<IFpgaService, FpgaService>();
            builder.Services.Configure<FpgaConfigurationService>(builder.Configuration.GetSection("FpgaConfigurationService"));
            builder.Services.AddDbContext<AppDbContext>(options => options.UseSqlite
            (builder.Configuration.GetConnectionString("DefaultDbConnection"),
            b => b.MigrationsAssembly("RsaWebapp")));
            builder.Services.AddAuthentication(CookieAuthenticationDefaults.AuthenticationScheme).
                AddCookie(x => x.LoginPath = "/login");
           

            builder.Services.AddAuthorization(options =>
            {
                options.AddPolicy("LoggedUserOnly", policy => policy.RequireClaim(ClaimTypes.Role,"LoggedUser"));
            });
            builder.Services.AddCascadingAuthenticationState();
            var app = builder.Build();

            // Configure the HTTP request pipeline.
            if (app.Environment.IsDevelopment())
            {
                app.UseWebAssemblyDebugging();
            }
            else
            {
                app.UseExceptionHandler("/Error");
            }
            app.MapControllers();
            app.UseStaticFiles();
            app.UseAntiforgery();
            app.UseSwagger();
            app.UseAuthorization();
            app.UseAuthorization();
            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "Blazor API V1");
            });
            app.MapRazorComponents<App>()
                .AddInteractiveServerRenderMode()
                .AddInteractiveWebAssemblyRenderMode()
                .AddAdditionalAssemblies(typeof(Client._Imports).Assembly);

            app.Run();
        }
    }
}
